/**
 * A simple GSM alarm based on off-the-shelf Siemens TC65
 * GSM/GPRS terminal.
 * 
 *  GSM-   GSM/GPRS  Siemens TC65.
 *      . 
 * 
 *          
 * ,       GPIO3. 
 * 
 *        GPIO4
 *     ,    , 
 *  " "       
 *  GPIO2,       .
 * 
 *        GPIO5
 *     ,    , 
 *  " "       
 *  GPIO2,       .
 * 
 *   GPIO1  ,   .
 * 
  :  ,   RS-232 (115200, 8, n, 1)
 * (, HyperTerminal, term95  minicom).
 * 
 * 
 * Author: "Dmitry" <dkorobkov@mail.ru>
 */


package gsmAlarm;

import javax.microedition.midlet.*;

import com.siemens.icm.io.*;

public class GSMAlarm extends MIDlet{
	//     
	final static int INPUT_BUTTON = 2; //  GPIO (GPIO3: 2), .  1
	final static int INPUT_DOOR = 3; //  GPIO (GPIO4: 3), .  2
	final static int INPUT_WINDOW = 4; //  GPIO (GPIO5: 4), .  3

	final static int OUTPUT_LED = 0; //  GPIO (GPIO1: 0), .  1
	final static int OUTPUT_SIREN = 1; //  GPIO (GPIO2: 1), .  2
	
	ATCommand		atCommand; //       .
	ATCommand		atCommandForTc65Io; //  -    I/O,  
	
	MyATCommandListener atCommandListener; //  -    . 
	MyATCommandListener atioCommandListener; //  65 (-)
	
	MenuSettings	menuSettings;
	Tc65Io			tc65Io;
	Tc65SerialPort	tc65SerialPort;
	SMSSendReceive  smsSendReceive;
	Indicator 		indicator;
	
	int HangupTimer = 0; //   
	
	boolean PinErrorMsg1Displayed = false; //      
	//  PIN   .
	boolean PinErrorMsg2Displayed = false;   
	  
	public GSMAlarm()
	{
	    try 
	    {
	        atCommand = new ATCommand(false);
	        atCommandForTc65Io = new ATCommand(false);
	        tc65SerialPort = new Tc65SerialPort();
	        //    -  GPIO,    
	        //     tc65Io. 
	        //       .
	        menuSettings = new MenuSettings(tc65SerialPort, atCommandForTc65Io);
	        tc65Io = new Tc65Io(atCommandForTc65Io, menuSettings);
	        atCommandListener = new MyATCommandListener(false, tc65Io, tc65SerialPort);
	        atioCommandListener = new MyATCommandListener(true, tc65Io, tc65SerialPort);
	        smsSendReceive = new SMSSendReceive(atCommand, menuSettings);
	        indicator = new Indicator(tc65Io, OUTPUT_LED, tc65SerialPort);
	        
	        //       .
	        menuSettings.smsSendReceive = smsSendReceive;
	        
	        //   atCommandListener    -,
	        //     ( RING)
//	        atCommand.addListener(atCommandListener);
	        //   URC,   - tc65Io
	        tc65Io.atCommand.addListener(atioCommandListener);
	        atCommand.addListener(atCommandListener);
	        
	        //     
	        menuSettings.start();
	        indicator.start();
	        
//	        System.out.println("InputStream and OutputStream opened");
	        serialOut("\n\r\n\rGSM alarm Java app started (build 090815).\n\r");
	    } 
	    catch(ATCommandFailedException e) 
	    {
	        serialOut(e.getMessage());
	        notifyDestroyed();
	    }
	    catch(Exception e) 
	    {
	        System.out.println(e);
	        notifyDestroyed();
	    }
	}

	private int DebugMsgCount = 3; //    
	
/**
 *   : ,  PIN...
 *  false     -   
 *  PIN  3.     .
 */	
	boolean init()
	{
		boolean bRet = true;
		
		try
		{
			menuSettings.DebugOut(atCommand.send("AT&F\r")); //   
			Thread.sleep(1000); //    

			//     PIN
			//    3,    , 
			// , , , PIN .
			String str = atCommand.send("AT^SPIC\r");
			menuSettings.DebugOut(str);
			//  -   ,  
			//  ERROR.
			int pos = str.indexOf("^SPIC:");
			int posError = str.indexOf("ERROR");
			char count = '2'; //  ,    
			if(pos != -1)
			{
				pos += 7;
				count = str.charAt(pos); 
//				serialOut("count="+count+"\n\r");
			}
			else
			{
				if(DebugMsgCount > 0) //    
				{
					serialOut("DEBUG(init()): "+str+"\n\r");
					DebugMsgCount--;
				}
			}
			
			//      3,   .
			//    .
			if(count == '3') 
			{
				menuSettings.DebugOut(atCommand.send("AT+CPIN="+ menuSettings.PINCode +"\r"));
				//     ,    ERROR,
				//    .     
				//     -.
				
				Thread.sleep(5000); //    
			
				//   ,     PIN
				String str2 = atCommand.send("AT^SPIC\r");
				menuSettings.DebugOut(str2);
				pos = str2.indexOf("^SPIC:");
				if(pos != -1)
				{
					pos += 7;
					if(str2.charAt(pos) == '3')
						bRet = true;
					else bRet = false;
				}
				else bRet = false;
			}
			else
			{
				if(PinErrorMsg1Displayed == false)
				{
					PinErrorMsg1Displayed = true;
					if(posError != -1)
						serialOut("ERROR: Something is wrong with SIM card (maybe not accepted by operator).\n\r");
					else
						serialOut("ERROR: PIN enter attempts is less than 3. Probably wrong PIN is used.\n\r");
				}
				bRet = false;
			}
		}
		catch(ATCommandFailedException e)
		{
			serialOut("ERROR: init failed\n\r");
//			serialOut(e.getMessage());
		}
		catch (InterruptedException e) 
		{
			serialOut("ERROR: init interrupted\n\r");
		}
		
		return bRet;
	}
	
	/**
	 * Wrappers for debug output to serial port
	 * @param str
	 */
	void serialOut(String str)
	{
		tc65SerialPort.serialOut(str.getBytes());
	}
	void serialOut(byte[] ar)
	{
		tc65SerialPort.serialOut(ar);
	}
	
	/**
	   * startApp()
	   */
	
	public void startApp() throws MIDletStateChangeException 
	{
		//   
        if(tc65Io.initIo())
        	menuSettings.DebugOut("\n\rI/O initialized.\n\r");
        else
        	menuSettings.DebugOut("\n\rERROR initialization I/O!\n\r");

		serialOut("\n\rSending PIN " + menuSettings.PINCode + "(wait 6 sec).\n\r");
		boolean bPinOK = false;
		
		while(bPinOK == false)
		{
			bPinOK = init();
			
	        if(bPinOK == false)
	        {
	        	if(PinErrorMsg2Displayed == false)
	        	{
	        		PinErrorMsg2Displayed = true;
		        	serialOut("PIN code " + menuSettings.PINCode + " is PROBABLY invalid. Change PIN and try again.\n\r");
	        	}
	    		indicator.SetMask(0xfffe);
	        }
	        
	        if(!menuSettings.isAlive()) //    
	        {
	        	destroyApp(true);
	        	return;
	        }
	        
	        try 
	        {
				Thread.sleep(10000);
			} 
	        catch (InterruptedException e) 
			{
	        	destroyApp(true);
	        	return;
			}
		}

		smsSendReceive.init();
        
		char FirstCharOfNumber = menuSettings.Number1.charAt(0);
		if(FirstCharOfNumber != '+' && 
				FirstCharOfNumber != '8')
			serialOut("Telephone number is wrong or not set, cannot start normally.\n\r"+
					"Use menu to enter and save telephone number.\r\n"+
					"Press ENTER to see menu.\r\n\r\n");		
		else
			serialOut("GSM alarm (build 0.14) is ready for operation.\n\rPress ENTER to see menu.\n\r\n\r");
		
		//    
		tc65Io.ResetAllInputs();
		
		//  ,       
		try {
			menuSettings.DebugOut(atCommand.send("AT+COLP=1\r"));
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ATCommandFailedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		int countLoops = 0;

		int STARTING = 1;
		int DISARMED = 2;
		int ARMED = 3;
		int state = STARTING;
		int prevstate = 0;
		
		indicator.SetMask(0x5555);
		while(menuSettings.isAlive()) 
		{
			try 
			{
				Thread.sleep(150);
				countLoops++;

				FirstCharOfNumber = menuSettings.Number1.charAt(0);  

				// :       -
				if(state != prevstate)
				{
					prevstate = state;
				}
				
				switch(state)
				{
				case 1:   // STARTING
					if(FirstCharOfNumber == '+' || 
							FirstCharOfNumber == '8')
					{
						state = DISARMED;
						indicator.SetMask(0x0001);
						if(menuSettings.sendSMSOnReboot)
							NotifyOnEvent(menuSettings.Number1, 
									"     .   .");
					}
					break;
				case 2: // DISARMED
					indicator.SetMask(0x0001); 
					
					if(tc65Io.GetChanged(INPUT_BUTTON))
					{
						menuSettings.DebugOut("\n\rButton pressed\n\r");
						
						indicator.SetMask(0x5555);
//						tc65SerialPort.serialOut("2armed");
						state = ARMED;
						Wait(menuSettings.DelayArmed, false);
						indicator.SetMask(0x0005);
						NotifyOnEvent(menuSettings.Number1, 
								"  .");
						tc65Io.ResetAllInputs(); //    
					}
					break;
				case 3: // ARMED
					if(tc65Io.GetChanged(INPUT_BUTTON))
					{
						menuSettings.DebugOut("\n\rButton pressed\n\r");
						tc65Io.ResetAllInputs();  
						state = DISARMED;
					}
					if(tc65Io.GetChanged(INPUT_DOOR))
					{
						menuSettings.DebugOut("\n\rDoor open\n\r");
						indicator.SetMask(0x3333);
						if(Wait(menuSettings.DelayAfterDoorOpen))
						{
							tc65Io.ResetAllInputs();
							state = DISARMED; //      
						}
						else
						{
							indicator.SetMask(0x5555);
							tc65Io.outIo(OUTPUT_SIREN, true);
							NotifyOnEvent(menuSettings.Number1, 
								" .");
							if(Wait(menuSettings.DelayRearm))
							{
								tc65Io.ResetAllInputs();
								state = DISARMED;
							}
							indicator.SetMask(0x0005);
							tc65Io.outIo(OUTPUT_SIREN, false); //
						}
					}
					if(tc65Io.GetChanged(INPUT_WINDOW))
					{
						menuSettings.DebugOut("\n\rWindow open\n\r");
						indicator.SetMask(0x5555);
						tc65Io.outIo(OUTPUT_SIREN, true);
						NotifyOnEvent(menuSettings.Number1, 
							" .");
						if(Wait(menuSettings.DelayRearm))
						{
							state = DISARMED;
							tc65Io.ResetAllInputs();
						}
						tc65Io.outIo(OUTPUT_SIREN, false);
						indicator.SetMask(0x0005);
					}
					break;
				default: //?
					state = STARTING;
				}

				ProcessUnsolicitedEvents();
				
//				if(countLoops == 10)
	//				tc65Io.RefreshInputs();
				
				if(countLoops == 20)
					smsSendReceive.PollNewSMS();
				
				if(countLoops >= 30)
					countLoops = 0;
				
				//   - ,    
				if(HangupTimer == 1)
				{
					menuSettings.DebugOut("Hanging up on timeout:"+atCommand.send("ATH\r"));
					HangupTimer = 0;
				}
				else if(HangupTimer > 0)
					HangupTimer--;
				
			} 
			catch(Exception ioe) 
			{
				serialOut("Excp "+ioe.getMessage()+">");
			}
		}
		
	    serialOut("\n\rJava app closed");
	    
	    destroyApp(true);
	  }

/**
 *   ,    ( RING  
 *    SMS)
*/
	void ProcessUnsolicitedEvents()
	{
  	  if(atCommandListener.hasNewEvent())
	  {
  		  menuSettings.DebugOut("\n\rURC: "+atCommandListener.strATCommand+"\n\r");
		  
  		  if(atCommandListener.strATCommand.indexOf("RING") != -1)
		  {
//			  serialOut("\n\rNew event: incoming ring\n\r");			  
		  }
		  else if(atCommandListener.strATCommand.indexOf("+CMTI:") != -1)
		  {
//			  serialOut("\n\rNew event: incoming SMS\n\r");
			  if(smsSendReceive.receiveSMS())
				  menuSettings.DebugOut("\n\rNew incoming SMS from " +
						  smsSendReceive.pdu.smsSenderID +
						  " at " + smsSendReceive.pdu.smsTime +
						  ": "+smsSendReceive.pdu.smsText+"\n\r");
		  }
		  else if(atCommandListener.strATCommand.indexOf("+COLP:") != -1)
		  {
			  //  ,    -  
			  try {
				atCommand.send("ATH\r");
			} catch (IllegalStateException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (ATCommandFailedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		  }
		  else
			  menuSettings.DebugOut("\n\rNew event (not processed): " + atCommandListener.strATCommand + "\n\r");			  
	  }
	}
	
	  /**
	   * pauseApp()
	   */
	  public void pauseApp() {
//	    System.out.println("RS232Demo: pauseApp()");
	  }

	  /**
	   * destroyApp()
	   *
	   * This is important.  It closes the app's RecordStore
	   * @param cond true if this is an unconditional destroy
	   *             false if it is not
	   *             currently ignored and treated as true
	   */
	  public void destroyApp(boolean cond) {
//		  System.out.println("RS232Demo: destroyApp(" + cond + ")");
		  //  ,   
		  indicator.bTerminateThread = true;
		  try {
			  indicator.join();
		  } catch (InterruptedException e) {
			  // TODO Auto-generated catch block
			  e.printStackTrace();
		  }

		  tc65Io.closeIo();
		  
		  menuSettings.close();
		  notifyDestroyed();
//		  tc65SerialPort.serialOut("destroyApp");
	  }
	  
/**
 *          -
 * @param num
 * @param msg
 */	  
	  private void NotifyOnEvent(String num, String msg)
	  {
		  //   ,     -
		  menuSettings.DebugOut("\n\rMessage: " + Translit(msg) + "\n\r");
		  
		  //   ,      SMS
		  smsSendReceive.sendSMS(num, msg);
		  
		  try {
			  menuSettings.DebugOut(atCommand.send("ATD"+num+";\r"));
			  HangupTimer = 50;
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ATCommandFailedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	  }
	  
	  /**
	   *      (   -)
	   * @param text
	   * @return
	   */
	  private String Translit(String text)
	  {
		  StringBuffer sb = new StringBuffer(text.length()+10);
		  for(int i=0; i<text.length(); i++)
		  {
			  char c = text.charAt(i);
			  switch(c)
			  {
			  case '': c = 'A';break; case '': c = 'a';break;
			  case '': c = 'B';break; case '': c = 'b';break;
			  case '': c = 'V';break; case '': c = 'v';break;
			  case '': c = 'G';break; case '': c = 'g';break;
			  case '': c = 'D';break; case '': c = 'd';break;
			  case '': c = 'E';break; case '': c = 'e';break;
			  case '': c = 'Z';break; case '': c = 'z';break;
			  case '': c = 'Z';break; case '': c = 'z';break;
			  case '': c = 'I';break; case '': c = 'i';break;
			  case '': c = 'J';break; case '': c = 'j';break;
			  case '': c = 'K';break; case '': c = 'k';break;
			  case '': c = 'L';break; case '': c = 'l';break;
			  case '': c = 'M';break; case '': c = 'm';break;
			  case '': c = 'N';break; case '': c = 'n';break;
			  case '': c = 'O';break; case '': c = 'o';break;
			  case '': c = 'P';break; case '': c = 'p';break;
			  case '': c = 'R';break; case '': c = 'r';break;
			  case '': c = 'S';break; case '': c = 's';break;
			  case '': c = 'T';break; case '': c = 't';break;
			  case '': c = 'U';break; case '': c = 'u';break;
			  case '': c = 'F';break; case '': c = 'f';break;
			  case '': c = 'H';break; case '': c = 'h';break;
			  case '': c = 'C';break; case '': c = 'c';break;
			  case '': c = 'H';break; case '': c = 'h';break;
			  case '': c = 'S';break; case '': c = 's';break;
			  case '': c = 'S';break; case '': c = 's';break;
			  case '': c = '\'';break; case '': c = '\'';break;
			  case '': c = 'Y';break; case '': c = 'y';break;
			  case '': c = '\'';break; case '': c = '\'';break;
			  case '': c = 'E';break; case '': c = 'e';break;
			  case '': c = 'U';break; case '': c = 'u';break;
			  case '': c = 'A';break; case '': c = 'a';break;
			  }
			  sb.append(c);
		  }
		  return sb.toString();
	  }
	  
/**
 *       
 * @param seconds -   
 * @return true,    
 */	  
	  private boolean Wait(int seconds)
	  {
		  return Wait(seconds, true);
	  }
	  
	  private boolean Wait(int seconds, boolean AbortOnButton)
	  {
		  boolean pressed = false;
		  
		  try 
		  {
			  
			  for(int i=0; i<seconds; i++)
			  {
				  Thread.sleep(1000);
//				  tc65SerialPort.serialOut('.');
				  if(AbortOnButton)
				  {
					  //   
					  if(tc65Io.GetChanged(INPUT_BUTTON))
					  {
						  i = seconds; // ,   
						  pressed = true;
					  }
				  }
			  }
		  } 
		  catch (InterruptedException e) 
		  {
			  // TODO Auto-generated catch block
			  e.printStackTrace();
		  }
		  
		  return pressed;
	  }
}
